@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Copyright © 2018 Damir Jelić <poljar@termina.org.uk>
 # Copyright © 2020-2021 Famedly GmbH
 #
@@ -23,34 +21,29 @@ In general these functions are not directly called. One should use an existing
 client like AsyncClient or HttpClient.
 """
 
-
-from __future__ import unicode_literals
+from __future__ import annotations
 
 import json
+import os
 from collections import defaultdict
+from collections.abc import Iterable
 from enum import Enum, unique
 from typing import (
     TYPE_CHECKING,
     Any,
     DefaultDict,
     Dict,
-    Iterable,
     List,
     Optional,
     Sequence,
-    Set,
     Tuple,
     Union,
 )
-
-from .exceptions import LocalProtocolError
+from uuid import UUID
 
 if TYPE_CHECKING:
     from .events.account_data import PushAction, PushCondition
 
-if False:
-    from uuid import UUID
-
 try:
     from urllib.parse import quote, urlencode, urlparse
 except ImportError:
@@ -59,8 +52,8 @@ except ImportError:
     from urlparse import urlparse  # type: ignore
 
 
-MATRIX_API_PATH = "/_matrix/client/r0"  # type: str
-MATRIX_MEDIA_API_PATH = "/_matrix/media/r0"  # type: str
+MATRIX_API_PATH: str = "/_matrix/client/r0"
+MATRIX_MEDIA_API_PATH: str = "/_matrix/media/r0"
 
 _FilterT = Union[None, str, Dict[Any, Any]]
 
@@ -145,14 +138,12 @@ class Api:
     """
 
     @staticmethod
-    def to_json(content_dict):
-        # type: (Dict[Any, Any]) -> str
+    def to_json(content_dict: Dict[Any, Any]) -> str:
         """Turn a dictionary into a json string."""
         return json.dumps(content_dict, separators=(",", ":"))
 
     @staticmethod
-    def to_canonical_json(content_dict):
-        # type: (Dict[Any, Any]) -> str
+    def to_canonical_json(content_dict: Dict[Any, Any]) -> str:
         """Turn a dictionary into a canonical json string."""
         return json.dumps(
             content_dict,
@@ -162,8 +153,7 @@ class Api:
         )
 
     @staticmethod
-    def mimetype_to_msgtype(mimetype):
-        # type: (str) -> str
+    def mimetype_to_msgtype(mimetype: str) -> str:
         """Turn a mimetype into a matrix message type."""
         if mimetype.startswith("image"):
             return "m.image"
@@ -175,8 +165,7 @@ class Api:
         return "m.file"
 
     @staticmethod
-    def mxc_to_http(mxc, homeserver=None):
-        # type: (str, Optional[str]) -> Optional[str]
+    def mxc_to_http(mxc: str, homeserver: Optional[str] = None) -> Optional[str]:
         """Convert a matrix content URI to a HTTP URI."""
         url = urlparse(mxc)
 
@@ -205,13 +194,12 @@ class Api:
     @staticmethod
     def encrypted_mxc_to_plumb(
         mxc,
-        key,
-        hash,
-        iv,
-        homeserver=None,
-        mimetype=None,
-    ):
-        # type: (str, str, str, str, Optional[str]) -> Optional[str]
+        key: str,
+        hash: str,
+        iv: str,
+        homeserver: Optional[str] = None,
+        mimetype: Optional[str] = None,
+    ) -> Optional[str]:
         """Convert a matrix content URI to a encrypted mxc URI.
 
         The return value of this function will have a URI schema of emxc://.
@@ -272,7 +260,9 @@ class Api:
 
     @staticmethod
     def _build_path(
-        path: List[str], query_parameters: Dict = None, base_path: str = MATRIX_API_PATH
+        path: List[str],
+        query_parameters: Optional[Dict] = None,
+        base_path: str = MATRIX_API_PATH,
     ) -> str:
         """Builds a percent-encoded path from a list of strings.
 
@@ -330,10 +320,11 @@ class Api:
 
     @staticmethod
     def register(
-        user,  # type: str
-        password=None,  # type: str
-        device_name="",  # type: Optional[str]
-        device_id="",  # type: Optional[str]
+        user: str,
+        password: Optional[str] = None,
+        device_name: Optional[str] = "",
+        device_id: Optional[str] = "",
+        auth_dict: Optional[dict[str, Any]] = None,
     ):
         """Register a new user.
 
@@ -346,13 +337,26 @@ class Api:
             device_id (str): ID of the client device. If this does not
                 correspond to a known client device, a new device will be
                 created.
+            auth_dict (Dict[str, Any, optional): The authentication dictionary
+                containing the elements for a particular registration flow.
+                If not provided, then m.login.dummy is used.
+                See the example below and here
+                https://spec.matrix.org/latest/client-server-api/#account-registration-and-management
+                for detailed documentation
+
+                Example:
+                        >>> auth_dict = {
+                        >>>     "type": "m.login.registration_token",
+                        >>>     "registration_token": "REGISTRATIONTOKEN",
+                        >>>     "session": "session-id-from-homeserver"
+                        >>> }
         """
         path = Api._build_path(["register"])
 
         content_dict = {
-            "auth": {"type": "m.login.dummy"},
             "username": user,
             "password": password,
+            "auth": auth_dict or {"type": "m.login.dummy"},
         }
 
         if device_id:
@@ -365,13 +369,12 @@ class Api:
 
     @staticmethod
     def login(
-        user,  # type: str
-        password=None,  # type: str
-        device_name="",  # type: Optional[str]
-        device_id="",  # type: Optional[str]
-        token=None,  # type: str
-    ):
-        # type: (...) -> Tuple[str, str, str]
+        user: str,
+        password: Optional[str] = None,
+        device_name: Optional[str] = "",
+        device_id: Optional[str] = "",
+        token: Optional[str] = None,
+    ) -> Tuple[str, str, str]:
         """Authenticate the user.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -386,6 +389,7 @@ class Api:
             device_id (str): ID of the client device. If this does not
                 correspond to a known client device, a new device will be
                 created.
+            token (str): Token for token-based login.
         """
         path = Api._build_path(path=["login"])
 
@@ -428,9 +432,8 @@ class Api:
 
     @staticmethod
     def login_raw(
-        auth_dict,  # type: Dict[str, Any]
-    ):
-        # type: (...) -> Tuple[str, str, str]
+        auth_dict: Dict[str, Any],
+    ) -> Tuple[str, str, str]:
         """Login to the homeserver using a raw dictionary.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -463,8 +466,8 @@ class Api:
 
     @staticmethod
     def logout(
-        access_token,  # type: str
-        all_devices=False,  # type: bool
+        access_token: str,
+        all_devices: bool = False,
     ):
         """Logout the session.
 
@@ -481,7 +484,7 @@ class Api:
         else:
             api_path = ["logout"]
 
-        content_dict = {}  # type: Dict
+        content_dict: Dict = {}
         return (
             "POST",
             Api._build_path(api_path, query_parameters),
@@ -493,11 +496,10 @@ class Api:
         access_token: str,
         since: Optional[str] = None,
         timeout: Optional[int] = None,
-        filter: _FilterT = None,
+        filter: Optional[_FilterT] = None,
         full_state: Optional[bool] = None,
         set_presence: Optional[str] = None,
-    ):
-        # type: (...) -> Tuple[str, str]
+    ) -> Tuple[str, str]:
         """Synchronise the client's state with the latest state on the server.
 
         Returns the HTTP method and HTTP path for the request.
@@ -547,13 +549,12 @@ class Api:
 
     @staticmethod
     def room_send(
-        access_token,  # type: str
-        room_id,  # type: str
-        event_type,  # type: str
-        body,  # type: Dict[Any, Any]
-        tx_id,  # type: Union[str, UUID]
-    ):
-        # type (...) -> Tuple[str, str, str]
+        access_token: str,
+        room_id: str,
+        event_type: str,
+        body: Dict[Any, Any],
+        tx_id: Union[str, UUID],
+    ) -> Tuple[str, str, str]:
         """Send a message event to a room.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -573,6 +574,63 @@ class Api:
 
         return ("PUT", Api._build_path(path, query_parameters), Api.to_json(body))
 
+    @staticmethod
+    def space_get_hierarchy(
+        access_token: str,
+        space_id: str,
+        from_page: Optional[str] = None,
+        limit: Optional[int] = None,
+        max_depth: Optional[int] = None,
+        suggested_only: bool = False,
+    ) -> Tuple[str, str]:
+        """Get rooms/spaces that are a part of the provided space.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            space_id (str): The ID of the space to get the hierarchy for.
+            from_page (str, optional): Pagination token from a previous request
+                to this endpoint.
+            limit (int, optional): The maximum number of rooms to return.
+            max_depth (int, optional): The maximum depth of the returned tree.
+            suggested_only (bool, optional): Whether to only return
+                rooms that are considered suggested. Defaults to False.
+        """
+        query_parameters = {"access_token": access_token}
+
+        if from_page:
+            query_parameters["from"] = from_page
+
+        if limit:
+            query_parameters["limit"] = limit
+
+        if max_depth:
+            query_parameters["max_depth"] = max_depth
+
+        if suggested_only:
+            query_parameters["suggested_only"] = suggested_only
+
+        path = ["rooms", space_id, "hierarchy"]
+
+        return ("GET", Api._build_path(path, query_parameters, "/_matrix/client/v1"))
+
+    @staticmethod
+    def direct_room_list(access_token: str, user_id: str) -> Tuple[str, str]:
+        """Lists all rooms flagged as direct the client is participating in.
+
+        Returns the HTTP method and HTTP path for the request.
+
+        Args:
+            access_token (str): The access token to be used within the request
+            user_id (str): The user id of the user to get the direct rooms for
+        """
+        query_parameters = {"access_token": access_token}
+
+        path = ["user", user_id, "account_data", "m.direct"]
+
+        return ("GET", Api._build_path(path, query_parameters))
+
     @staticmethod
     def room_get_event(
         access_token: str, room_id: str, event_id: str
@@ -594,13 +652,12 @@ class Api:
 
     @staticmethod
     def room_put_state(
-        access_token,  # type str
-        room_id,  # type str
-        event_type,  # type str
-        body,  # type Dict[Any, Any]
-        state_key="",  # type str
-    ):
-        # type (...) -> Tuple[str, str, str]
+        access_token: str,
+        room_id: str,
+        event_type: str,
+        body: Dict[Any, Any],
+        state_key: str = "",
+    ) -> Tuple[str, str, str]:
         """Send a state event.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -622,8 +679,9 @@ class Api:
         return ("PUT", Api._build_path(path, query_parameters), Api.to_json(body))
 
     @staticmethod
-    def room_get_state_event(access_token, room_id, event_type, state_key=""):
-        # type (str, str, str) -> Tuple[str, str]
+    def room_get_state_event(
+        access_token, room_id: str, event_type: str, state_key: str = ""
+    ) -> Tuple[str, str]:
         """Fetch a state event.
 
         Returns the HTTP method and HTTP path for the request.
@@ -643,8 +701,7 @@ class Api:
         return ("GET", Api._build_path(path, query_parameters))
 
     @staticmethod
-    def room_get_state(access_token, room_id):
-        # type (str, str) -> Tuple[str, str]
+    def room_get_state(access_token: str, room_id: str) -> Tuple[str, str]:
         """Fetch the current state for a room.
 
         Returns the HTTP method and HTTP path for the request.
@@ -662,13 +719,12 @@ class Api:
 
     @staticmethod
     def room_redact(
-        access_token,  # type: str
-        room_id,  # type: str
-        event_id,  # type: str
-        tx_id,  # type: Union[str, UUID]
-        reason=None,  # type: Optional[str]
-    ):
-        # type (...) -> Tuple[str, str, str]
+        access_token: str,
+        room_id: str,
+        event_id: str,
+        tx_id: Union[str, UUID],
+        reason: Optional[str] = None,
+    ) -> Tuple[str, str, str]:
         """Strip information out of an event.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -694,8 +750,9 @@ class Api:
         return ("PUT", Api._build_path(path, query_parameters), Api.to_json(body))
 
     @staticmethod
-    def room_kick(access_token, room_id, user_id, reason=None):
-        # type (str, str, str, Optional[str]) -> Tuple[str, str, str]
+    def room_kick(
+        access_token: str, room_id: str, user_id: str, reason: Optional[str] = None
+    ) -> Tuple[str, str, str]:
         """Kick a user from a room, or withdraw their invitation.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -808,8 +865,9 @@ class Api:
         )
 
     @staticmethod
-    def room_invite(access_token, room_id, user_id):
-        # type (str, str, str) -> Tuple[str, str, str]
+    def room_invite(
+        access_token: str, room_id: str, user_id: str
+    ) -> Tuple[str, str, str]:
         """Invite a user to a room.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -828,22 +886,22 @@ class Api:
 
     @staticmethod
     def room_create(
-        access_token,  # type: str
-        visibility=RoomVisibility.private,  # type: RoomVisibility
-        alias=None,  # type: Optional[str]
-        name=None,  # type: Optional[str]
-        topic=None,  # type: Optional[str]
-        room_version=None,  # type: Optional[str]
-        federate=True,  # type: bool
-        is_direct=False,  # type: bool
-        preset=None,  # type: Optional[RoomPreset]
-        invite=(),  # type: Sequence[str]
-        initial_state=(),  # type: Sequence[Dict[str, Any]]
-        power_level_override=None,  # type: Optional[Dict[str, Any]]
-        predecessor=None,  # type: Optional[Dict[str, Any]]
-        space=False,  # type: bool
-    ):
-        # type (...) -> Tuple[str, str, str]
+        access_token: str,
+        visibility: RoomVisibility = RoomVisibility.private,
+        alias: Optional[str] = None,
+        name: Optional[str] = None,
+        topic: Optional[str] = None,
+        room_version: Optional[str] = None,
+        room_type: Optional[str] = None,
+        federate: bool = True,
+        is_direct: bool = False,
+        preset: Optional[RoomPreset] = None,
+        invite: Sequence[str] = (),
+        initial_state: Sequence[Dict[str, Any]] = (),
+        power_level_override: Optional[Dict[str, Any]] = None,
+        predecessor: Optional[Dict[str, Any]] = None,
+        space: bool = False,
+    ) -> Tuple[str, str, str]:
         """Create a new room.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -869,6 +927,12 @@ class Api:
                 If a version not supported by the homeserver is specified,
                 a 400 ``M_UNSUPPORTED_ROOM_VERSION`` error will be returned.
 
+            room_type (str, optional): The room type to set.
+                If not specified, the homeserver will use its default setting.
+                In spec v1.2 the following room types are specified:
+                    - ``m.space``
+                Unspecified room types are permitted through the use of Namespaced Identifiers.
+
             federate (bool): Whether to allow users from other homeservers from
                 joining the room. Defaults to ``True``.
                 Cannot be changed later.
@@ -922,6 +986,9 @@ class Api:
         if room_version:
             body["room_version"] = room_version
 
+        if room_type:
+            body["creation_content"]["type"] = room_type
+
         if preset:
             body["preset"] = preset.value
 
@@ -943,8 +1010,7 @@ class Api:
         return ("POST", Api._build_path(path, query_parameters), Api.to_json(body))
 
     @staticmethod
-    def join(access_token, room_id):
-        # type (str, str) -> Tuple[str, str, str]
+    def join(access_token: str, room_id: str) -> Tuple[str, str, str]:
         """Join a room.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -960,8 +1026,7 @@ class Api:
         return ("POST", Api._build_path(path, query_parameters), Api.to_json(body))
 
     @staticmethod
-    def room_leave(access_token, room_id):
-        # type (str, str) -> Tuple[str, str, str]
+    def room_leave(access_token: str, room_id: str) -> Tuple[str, str, str]:
         """Leave a room.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -977,8 +1042,7 @@ class Api:
         return ("POST", Api._build_path(path, query_parameters), Api.to_json(body))
 
     @staticmethod
-    def room_forget(access_token, room_id):
-        # type (str, str) -> Tuple[str, str, str]
+    def room_forget(access_token: str, room_id: str) -> Tuple[str, str, str]:
         """Forget a room.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1002,8 +1066,7 @@ class Api:
         direction: MessageDirection = MessageDirection.back,
         limit: int = 10,
         message_filter: Optional[Dict[Any, Any]] = None,
-    ):
-        # type (...) -> Tuple[str, str]
+    ) -> Tuple[str, str]:
         """Get room messages.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1052,8 +1115,9 @@ class Api:
         return "GET", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def keys_upload(access_token, key_dict):
-        # type: (str, Dict[str, Any]) -> Tuple[str, str, str]
+    def keys_upload(
+        access_token: str, key_dict: Dict[str, Any]
+    ) -> Tuple[str, str, str]:
         """Publish end-to-end encryption keys.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1070,8 +1134,9 @@ class Api:
         return ("POST", Api._build_path(path, query_parameters), Api.to_json(body))
 
     @staticmethod
-    def keys_query(access_token, user_set, token=None):
-        # type: (str, Iterable[str], Optional[str]) -> Tuple[str, str, str]
+    def keys_query(
+        access_token: str, user_set: Iterable[str], token: Optional[str] = None
+    ) -> Tuple[str, str, str]:
         """Query the current devices and identity keys for the given users.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1088,9 +1153,9 @@ class Api:
         query_parameters = {"access_token": access_token}
         path = ["keys", "query"]
 
-        content = {
+        content: Dict[str, Dict[str, List]] = {
             "device_keys": {user: [] for user in user_set}
-        }  # type: Dict[str, Dict[str, List]]
+        }
 
         if token:
             content["token"] = token  # type: ignore
@@ -1098,8 +1163,9 @@ class Api:
         return ("POST", Api._build_path(path, query_parameters), Api.to_json(content))
 
     @staticmethod
-    def keys_claim(access_token, user_set):
-        # type: (str, Dict[str, Iterable[str]]) -> Tuple[str, str, str]
+    def keys_claim(
+        access_token: str, user_set: Dict[str, Iterable[str]]
+    ) -> Tuple[str, str, str]:
         """Claim one-time keys for use in Olm pre-key messages.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1113,7 +1179,7 @@ class Api:
         query_parameters = {"access_token": access_token}
         path = ["keys", "claim"]
 
-        payload = defaultdict(dict)  # type: DefaultDict[str, Dict[str, str]]
+        payload: DefaultDict[str, Dict[str, str]] = defaultdict(dict)
 
         for user_id, device_list in user_set.items():
             for device_id in device_list:
@@ -1125,12 +1191,11 @@ class Api:
 
     @staticmethod
     def to_device(
-        access_token,  # type: str
-        event_type,  # type: str
-        content,  # type: Dict[Any, Any]
-        tx_id,  # type: Union[str, UUID]
-    ):
-        # type: (...) -> Tuple[str, str, str]
+        access_token: str,
+        event_type: str,
+        content: Dict[Any, Any],
+        tx_id: Union[str, UUID],
+    ) -> Tuple[str, str, str]:
         r"""Send to-device events to a set of client devices.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1149,8 +1214,7 @@ class Api:
         return ("PUT", Api._build_path(path, query_parameters), Api.to_json(content))
 
     @staticmethod
-    def devices(access_token):
-        # type: (str) -> Tuple[str, str]
+    def devices(access_token: str) -> Tuple[str, str]:
         """Get the list of devices for the current user.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1183,11 +1247,10 @@ class Api:
 
     @staticmethod
     def delete_devices(
-        access_token,  # type: str
-        devices,  # type: List[str]
-        auth_dict=None,  # type: Optional[Dict[str, str]]
-    ):
-        # type: (...) -> Tuple[str, str, str]
+        access_token: str,
+        devices: List[str],
+        auth_dict: Optional[Dict[str, str]] = None,
+    ) -> Tuple[str, str, str]:
         """Delete a device.
 
         This API endpoint uses the User-Interactive Authentication API.
@@ -1208,7 +1271,7 @@ class Api:
         query_parameters = {"access_token": access_token}
         path = ["delete_devices"]
 
-        content = {"devices": devices}  # type: Dict[str, Any]
+        content: Dict[str, Any] = {"devices": devices}
 
         if auth_dict:
             content["auth"] = auth_dict
@@ -1216,8 +1279,7 @@ class Api:
         return ("POST", Api._build_path(path, query_parameters), Api.to_json(content))
 
     @staticmethod
-    def joined_members(access_token, room_id):
-        # type: (str, str) -> Tuple[str, str]
+    def joined_members(access_token: str, room_id: str) -> Tuple[str, str]:
         """Get the list of joined members for a room.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1232,8 +1294,7 @@ class Api:
         return "GET", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def joined_rooms(access_token):
-        # type: (str) -> Tuple[str, str]
+    def joined_rooms(access_token: str) -> Tuple[str, str]:
         """Get the list of joined rooms for the logged in account.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1247,8 +1308,7 @@ class Api:
         return "GET", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def room_resolve_alias(room_alias):
-        # type: (str) -> Tuple[str, str]
+    def room_resolve_alias(room_alias: str) -> Tuple[str, str]:
         """Resolve a room alias to a room ID.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1261,8 +1321,7 @@ class Api:
         return "GET", Api._build_path(path)
 
     @staticmethod
-    def room_delete_alias(access_token, room_alias):
-        # type: (str, str) -> Tuple[str, str]
+    def room_delete_alias(access_token: str, room_alias: str) -> Tuple[str, str]:
         """Delete a room alias.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1277,8 +1336,9 @@ class Api:
         return "DELETE", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def room_put_alias(access_token, room_alias, room_id):
-        # type: (str, str, str) -> Tuple[str, str, str]
+    def room_put_alias(
+        access_token: str, room_alias: str, room_id: str
+    ) -> Tuple[str, str, str]:
         """Add a room alias.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1297,8 +1357,7 @@ class Api:
         return "PUT", Api._build_path(path, query_parameters), Api.to_json(body)
 
     @staticmethod
-    def room_get_visibility(room_id):
-        # type: (str) -> Tuple[str, str]
+    def room_get_visibility(room_id: str) -> Tuple[str, str]:
         """Get visibility of a room in the directory.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1312,13 +1371,12 @@ class Api:
 
     @staticmethod
     def room_typing(
-        access_token,  # type: str
-        room_id,  # type: str
-        user_id,  # type: str
-        typing_state=True,  # type: bool
-        timeout=30000,  # type: int
-    ):
-        # type: (...) -> Tuple[str, str, str]
+        access_token: str,
+        room_id: str,
+        user_id: str,
+        typing_state: bool = True,
+        timeout: int = 30000,
+    ) -> Tuple[str, str, str]:
         """Send a typing notice to the server.
 
         This tells the server that the user is typing for the next N
@@ -1371,12 +1429,11 @@ class Api:
 
     @staticmethod
     def room_read_markers(
-        access_token,  # type: str
-        room_id,  # type: str
-        fully_read_event,  # type: str
-        read_event=None,  # type: Optional[str]
-    ):
-        # type: (...) -> Tuple[str, str, str]
+        access_token: str,
+        room_id: str,
+        fully_read_event: str,
+        read_event: Optional[str] = None,
+    ) -> Tuple[str, str, str]:
         """Update fully read marker and optionally read marker for a room.
 
         This sets the position of the read marker for a given room,
@@ -1422,10 +1479,9 @@ class Api:
 
     @staticmethod
     def upload(
-        access_token,  # type: str
-        filename=None,  # type: Optional[str]
-    ):
-        # type: (...) -> Tuple[str, str, str]
+        access_token: str,
+        filename: Optional[str] = None,
+    ) -> Tuple[str, str, str]:
         """Upload a file's content to the content repository.
 
         Returns the HTTP method, HTTP path and empty data for the request.
@@ -1452,12 +1508,12 @@ class Api:
 
     @staticmethod
     def download(
-        server_name,  # type: str
-        media_id,  # type: str
-        filename=None,  # type: Optional[str]
-        allow_remote=True,  # type: bool
-    ):
-        # type: (...) -> Tuple[str, str]
+        server_name: str,
+        media_id: str,
+        filename: Optional[str] = None,
+        allow_remote: bool = True,
+        file: Optional[os.PathLike] = None,
+    ) -> Tuple[str, str]:
         """Get the content of a file from the content repository.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1472,6 +1528,7 @@ class Api:
                 attempt to fetch the media if it is deemed remote.
                 This is to prevent routing loops where the server contacts
                 itself.
+            file (os.PathLike): The file to stream the downloaded content to.
         """
         query_parameters = {
             "allow_remote": "true" if allow_remote else "false",
@@ -1485,14 +1542,13 @@ class Api:
 
     @staticmethod
     def thumbnail(
-        server_name,  # type: str
-        media_id,  # type: str
-        width,  # type: int
-        height,  # type: int
+        server_name: str,
+        media_id: str,
+        width: int,
+        height: int,
         method=ResizingMethod.scale,  # ŧype: ResizingMethod
-        allow_remote=True,  # type: bool
-    ):
-        # type: (...) -> Tuple[str, str]
+        allow_remote: bool = True,
+    ) -> Tuple[str, str]:
         """Get the thumbnail of a file from the content repository.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1521,7 +1577,9 @@ class Api:
         return ("GET", Api._build_path(path, query_parameters, MATRIX_MEDIA_API_PATH))
 
     @staticmethod
-    def profile_get(user_id: str, access_token: str = None) -> Tuple[str, str]:
+    def profile_get(
+        user_id: str, access_token: Optional[str] = None
+    ) -> Tuple[str, str]:
         """Get the combined profile information for a user.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1542,8 +1600,9 @@ class Api:
         return "GET", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def profile_get_displayname(user_id, access_token: str = None):
-        # type (str, str) -> Tuple[str, str]
+    def profile_get_displayname(
+        user_id: str, access_token: Optional[str] = None
+    ) -> Tuple[str, str]:
         """Get display name.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1562,8 +1621,9 @@ class Api:
         return "GET", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def profile_set_displayname(access_token, user_id, display_name):
-        # type (str, str, str) -> Tuple[str, str, str]
+    def profile_set_displayname(
+        access_token: str, user_id: str, display_name: str
+    ) -> Tuple[str, str, str]:
         """Set display name.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1580,8 +1640,9 @@ class Api:
         return ("PUT", Api._build_path(path, query_parameters), Api.to_json(content))
 
     @staticmethod
-    def profile_get_avatar(user_id, access_token: str = None):
-        # type (str, str) -> Tuple[str, str]
+    def profile_get_avatar(
+        user_id: str, access_token: Optional[str] = None
+    ) -> Tuple[str, str]:
         """Get avatar URL.
 
         Returns the HTTP method and HTTP path for the request.
@@ -1599,8 +1660,9 @@ class Api:
         return "GET", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def profile_set_avatar(access_token, user_id, avatar_url):
-        # type (str, str, str) -> Tuple[str, str, str]
+    def profile_set_avatar(
+        access_token: str, user_id: str, avatar_url: str
+    ) -> Tuple[str, str, str]:
         """Set avatar url.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1636,7 +1698,7 @@ class Api:
 
     @staticmethod
     def set_presence(
-        access_token: str, user_id: str, presence: str, status_msg: str = None
+        access_token: str, user_id: str, presence: str, status_msg: Optional[str] = None
     ):
         """This API sets the given user's presence state.
 
@@ -1657,8 +1719,7 @@ class Api:
         return ("PUT", Api._build_path(path, query_parameters), Api.to_json(content))
 
     @staticmethod
-    def whoami(access_token):
-        # type (str) -> Tuple[str, str]
+    def whoami(access_token: str) -> Tuple[str, str]:
         """Get information about the owner of a given access token.
 
         Returns the HTTP method, HTTP path and data for the request.
@@ -1672,8 +1733,9 @@ class Api:
         return "GET", Api._build_path(path, query_parameters)
 
     @staticmethod
-    def room_context(access_token, room_id, event_id, limit=None):
-        # type (str) -> Tuple[str, str]
+    def room_context(
+        access_token: str, room_id: str, event_id: str, limit: Optional[int] = None
+    ) -> Tuple[str, str]:
         """Fetch a number of events that happened before and after an event.
         This allows clients to get the context surrounding an event.
 
@@ -1783,8 +1845,8 @@ class Api:
         rule_id: str,
         before: Optional[str] = None,
         after: Optional[str] = None,
-        actions: Sequence["PushAction"] = (),
-        conditions: Optional[Sequence["PushCondition"]] = None,
+        actions: Sequence[PushAction] = (),
+        conditions: Optional[Sequence[PushCondition]] = None,
         pattern: Optional[str] = None,
     ) -> Tuple[str, str, str]:
         """Create or modify an existing user-created push rule.
@@ -1922,7 +1984,7 @@ class Api:
         scope: str,
         kind: PushRuleKind,
         rule_id: str,
-        actions: Sequence["PushAction"],
+        actions: Sequence[PushAction],
     ) -> Tuple[str, str, str]:
         """Set the actions for an existing built-in or user-created push rule.
 
@@ -1987,7 +2049,7 @@ class Api:
 
         query_parameters = {"access_token": access_token}
         path = ["directory", "room", alias]
-        content = dict()
+        content = {}
         content["room_id"] = room_id
 
         return (
