@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Copyright © 2018 Damir Jelić <poljar@termina.org.uk>
 # Copyright © 2020-2021 Famedly GmbH
 #
@@ -15,7 +13,6 @@
 # CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 # CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-from __future__ import unicode_literals
 
 import re
 
@@ -37,16 +34,16 @@ def extend_with_default(validator_class):
             if "default" in subschema:
                 instance.setdefault(property, subschema["default"])
 
-        for error in validate_properties(validator, properties, instance, schema):
-            yield error
+        yield from validate_properties(validator, properties, instance, schema)
 
     return validators.extend(validator_class, {"properties": set_defaults})
 
 
 Validator = extend_with_default(Draft4Validator)
+Checker = FormatChecker()
 
 
-@FormatChecker.cls_checks("user_id", ValueError)
+@Checker.checks("user_id", ValueError)
 def check_user_id(value: str) -> bool:
     if not value.startswith("@"):
         raise ValueError("UserIDs start with @")
@@ -57,7 +54,7 @@ def check_user_id(value: str) -> bool:
     return True
 
 
-@FormatChecker.cls_checks("http_url", ValueError)
+@Checker.checks("http_url", ValueError)
 def check_http_url(value: str) -> bool:
     if not re.match(r"^https?://.+", value):
         raise ValueError("Must be http://... or https://... URL")
@@ -66,7 +63,7 @@ def check_http_url(value: str) -> bool:
 
 
 def validate_json(instance, schema):
-    Validator(schema, format_checker=FormatChecker()).validate(instance)
+    Validator(schema, format_checker=Checker).validate(instance)
 
 
 class Schemas:
@@ -245,6 +242,32 @@ class Schemas:
         "required": ["user_id", "device_id", "access_token"],
     }
 
+    register_flows = {
+        "type": "object",
+        "properties": {
+            "flows": {
+                "type": "array",
+                "items": {
+                    "type": "object",
+                    "properties": {
+                        "stages": {
+                            "type": "array",
+                            "items": {"type": "string"},
+                        }
+                    },
+                    "required": ["stages"],
+                },
+            },
+            "params": {"type": "object"},
+            "session": {"type": "string"},
+            "completed": {"type": "array", "items": {"type": "string"}},
+            "user_id": {"type": "string", "format": "user_id"},
+            "device_id": {"type": "string"},
+            "access_token": {"type": "string"},
+        },
+        "required": ["flows", "params", "session"],
+    }
+
     login = {
         "type": "object",
         "properties": {
@@ -776,6 +799,7 @@ class Schemas:
                     "creator": {"type": "string", "format": "user_id"},
                     "m.federate": {"type": "boolean", "default": True},
                     "room_version": {"type": "string", "default": "1"},
+                    "type": {"type": "string", "default": ""},
                     "predecessor": {
                         "type": "object",
                         "properties": {
@@ -874,7 +898,7 @@ class Schemas:
             "type": {"type": "string"},
             "content": {
                 "type": "object",
-                "properties": {"alias": {"type": "string"}},
+                "properties": {"alias": {"type": ["string", "null"]}},
                 "required": [],
             },
         },
@@ -936,7 +960,6 @@ class Schemas:
                     "canonical": {"type": "boolean", "default": False},
                     "via": {"type": "array", "items": {"type": "string"}},
                 },
-                "required": ["via"],
             },
         },
         "required": ["type", "sender", "content", "state_key"],
@@ -955,7 +978,6 @@ class Schemas:
                     "via": {"type": "array", "items": {"type": "string"}},
                     "order": {"type": "string"},
                 },
-                "required": ["via"],
             },
         },
         "required": ["type", "sender", "content", "state_key"],
@@ -978,7 +1000,7 @@ class Schemas:
                     },
                     "url": {"type": "string"},
                 },
-                "required": ["url"],
+                "required": [],
             },
         },
         "required": ["type", "sender", "content", "state_key"],
@@ -1087,6 +1109,35 @@ class Schemas:
         "required": ["sender"],
     }
 
+    reaction = {
+        "type": "object",
+        "properties": {
+            "sender": {
+                "type": "string",
+                "format": "user_id",
+            },
+            "content": {
+                "type": "object",
+                "properties": {
+                    "m.relates_to": {
+                        "type": "object",
+                        "properties": {
+                            "rel_type": {
+                                "type": "string",
+                                "const": "m.annotation",
+                            },
+                            "event_id": {"type": "string"},
+                            "key": {"type": "string"},
+                        },
+                        "required": ["rel_type", "event_id", "key"],
+                    },
+                },
+                "required": ["m.relates_to"],
+            },
+        },
+        "required": ["sender", "content"],
+    }
+
     room_resolve_alias = {
         "type": "object",
         "properties": {
@@ -1925,3 +1976,65 @@ class Schemas:
             "state_key",
         ],
     }
+
+    space_hierarchy = {
+        "type": "object",
+        "properties": {
+            "next_batch": {"type": "string"},
+            "rooms": {
+                "type": "array",
+                "items": {
+                    "type": "object",
+                    "properties": {
+                        "avatar_url": {"type": "string"},
+                        "canonical_alias": {"type": "string"},
+                        "children_state": {
+                            "type": "array",
+                            "items": {
+                                "type": "object",
+                                "properties": {
+                                    "content": {
+                                        "type": "object",
+                                        "properties": {
+                                            "via": {
+                                                "type": "array",
+                                                "items": {"type": "string"},
+                                            },
+                                        },
+                                        "required": ["via"],
+                                    },
+                                    "origin_server_ts": {"type": "integer"},
+                                    "sender": {"type": "string"},
+                                    "state_key": {"type": "string"},
+                                    "type": {"type": "string"},
+                                },
+                                "required": [
+                                    "content",
+                                    "origin_server_ts",
+                                    "sender",
+                                    "state_key",
+                                    "type",
+                                ],
+                            },
+                        },
+                        "guest_can_join": {"type": "boolean"},
+                        "join_rule": {"type": "string"},
+                        "name": {"type": "string"},
+                        "num_joined_members": {"type": "integer"},
+                        "room_id": {"type": "string"},
+                        "room_type": {"type": "string"},
+                        "topic": {"type": "string"},
+                        "world_readable": {"type": "boolean"},
+                    },
+                    "required": [
+                        "children_state",
+                        "guest_can_join",
+                        "num_joined_members",
+                        "room_id",
+                        "world_readable",
+                    ],
+                },
+            },
+        },
+        "required": ["rooms"],
+    }
